"""Set a random background image (XWin)

Select a random image from a (list of) directory(s)
and set it as the desktop wallpaper (display it in the root window)
using xli program.

"""

from fcntl import flock, LOCK_EX, LOCK_UN, LOCK_NB
from datetime import timedelta
import os
import random
import re
import subprocess
import sys
from time import time

from xsetbg_conf import xsetbg_dir, xsetbg_conf
from xsetbg_db import xsetbg_db

__all__ = ['change']


def error(error_str):
    sys.exit("%s: Error: %s\n" % (sys.argv[0], error_str))


if not xsetbg_db:
    error("No database found. Run rescan_fs.py. Abort.")

if xsetbg_db.select().count() == 0:
    error("No images found. Run rescan_fs.py. Abort.")


# minimum time in seconds between background image changes
if xsetbg_conf.has_option("xsetbg", "min_pause"):
    min_pause = xsetbg_conf.getint("xsetbg", "min_pause")
else:
    min_pause = 60

borders = xsetbg_conf.get("xsetbg", "borders").split(',')
if xsetbg_conf.has_option("xsetbg", "borders"):
    borders = [border.strip() for border in
               xsetbg_conf.get("xsetbg", "borders").split(',')]
else:
    borders = ["darkcyan", "steelblue", "midnightblue"]

# minimum time in seconds between occurences of the same image
if xsetbg_conf.has_option("xsetbg", "min_delay"):
    min_delay = xsetbg_conf.get("xsetbg", "min_delay")

    # Borrowed from http://stackoverflow.com/a/2765366
    td_re = re.compile('(?:(?P<years>\d+)y)?'
                       '(?:(?P<months>\d+)m)?'
                       '(?:(?P<days>\d+)d)?'
                       '(?:T(?:(?P<hours>\d+)h)?'
                       '(?:(?P<minutes>\d+)m)?(?:(?P<seconds>\d+)s)?)?')
    td_dict = td_re.match(min_delay).groupdict(0)
    delta = timedelta(days=int(td_dict['days']) +
                      (int(td_dict['months']) * 30) +
                      (int(td_dict['years']) * 365),
                      hours=int(td_dict['hours']),
                      minutes=int(td_dict['minutes']),
                      seconds=int(td_dict['seconds']))

    if delta:
        min_delay = delta.days * 24*3600 + delta.seconds
    else:
        min_delay = int(min_delay)

else:
    min_delay = 3600*24  # 24 hours


if xsetbg_db.select('last_shown IS NULL OR last_shown < %d' %
                    (time() - min_delay)).count() == 0:
    error("No unshown images found. Run rescan_fs.py "
          "or decrease min_delay. Abort.")

fs_encoding = xsetbg_conf.get("images", "fs_encoding")

def change(force=False):
    # Use the program's file as the lock file:
    # lock it to prevent two processes run in parallel.
    lock_file = open(os.path.join(xsetbg_dir, 'xsetbg.py'), 'r')

    try:
        flock(lock_file, LOCK_EX | LOCK_NB)
    except IOError:  # already locked
        lock_file.close()
        return

    try:
        timestamp = xsetbg_db.select('last_shown IS NOT NULL',
                                     orderBy='-last_shown')[0].last_shown
        current_time = time()

        if not force and timestamp is not None and \
                current_time - timestamp < min_pause:
            # Too early to change background
            return

        # Select a random image that has never been shown
        not_shown_select = xsetbg_db.select('last_shown IS NULL')
        not_shown_count = not_shown_select.count()
        if not_shown_count:
            row = not_shown_select[random.randint(0, not_shown_count - 1)]
        else:
            old_shown_select = xsetbg_db.select(
                'last_shown IS NOT NULL AND last_shown < %d' %
                current_time - min_delay)
            old_shown_count = old_shown_select.count()
            if old_shown_count:
                row = old_shown_select[random.randint(0, old_shown_count - 1)]
            else:
                error("No images to show found. Run rescan_fs.py "
                      "or decrease min_delay. Abort.")

        program_options = ["xli", "-border", random.choice(borders),
                           "-center", "-onroot", "-quiet", "-zoom", "auto",
                           row.full_name.encode(fs_encoding)]

        rc = subprocess.call(program_options)
        if rc:
            error("cannot execute xli!")
        else:
            row.last_shown = current_time

    finally:
        # Unlock and close the lock file
        flock(lock_file, LOCK_UN)
        lock_file.close()
